@@ -1,4 +1,19 @@
 # -*- coding: utf-8 -*-
+#       This program is free software; you can redistribute it and/or modify
+#       it under the terms of the GNU General Public License as published by
+#       the Free Software Foundation; either version 2 of the License, or
+#       (at your option) any later version.
+#
+#       This program is distributed in the hope that it will be useful,
+#       but WITHOUT ANY WARRANTY; without even the implied warranty of
+#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#       GNU General Public License for more details.
+#
+#       You should have received a copy of the GNU General Public License
+#       along with this program; if not, write to the Free Software
+#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+#       MA 02110-1301, USA.
+
 """
 Once you've created features within Google Earth and examined the KML
 code Google Earth generates, you'll notice how styles are an important
@@ -9,52 +24,39 @@ import logging
 logger = logging.getLogger('fastkml.styles')
 
 import config
-
-try:
-    from lxml import etree
-    LXML = True
-except ImportError:
-    import xml.etree.ElementTree as etree
-    LXML = False
+from config import etree
+from base import _BaseObject
 
 
-class StyleUrl(object):
+
+
+class StyleUrl(_BaseObject):
     """
     URL of a <Style> or <StyleMap> defined in a Document. If the style
     is in the same file, use a # reference. If the style is defined in
     an external file, use a full URL along with # referencing.
     """
+    __name__ = 'styleUrl'
     url = None
 
-    def __init__(self, ns=None, url=None):
+    def __init__(self, ns=None, id=None, url=None):
+        super(StyleUrl, self).__init__(ns, id)
         self.url = url
-        if ns == None:
-            self.ns = config.NS
-        else:
-            self.ns = ns
 
     def etree_element(self):
         if self.url:
-            element = etree.Element(self.ns + "styleUrl")
+            element = super(StyleUrl, self).etree_element()
             element.text = self.url
             return element
         else:
-            logger.critical('No url given for styleUrl')
-            raise ValueError
-
-    def from_string(self, xml_string):
-        self.from_element(etree.XML(xml_string))
+            raise ValueError('No url given for styleUrl')
 
     def from_element(self, element):
-        if self.ns + "styleUrl" != element.tag:
-            raise TypeError
-        else:
-            self.url = element.text
+        super(StyleUrl, self).from_element(element)
+        self.url = element.text
 
 
-
-
-class _StyleSelector(object):
+class _StyleSelector(_BaseObject):
     """
     This is an abstract element and cannot be used directly in a KML file.
     It is the base type for the <Style> and <StyleMap> elements. The
@@ -62,31 +64,7 @@ class _StyleSelector(object):
     Placemark. An element derived from StyleSelector is uniquely identified
     by its id and its url.
     """
-    id = None
-    ns = None
 
-    def __init__(self, ns=None, id=None):
-        self.id = id
-        if ns == None:
-            self.ns = config.NS
-        else:
-            self.ns = ns
-
-    def etree_element(self):
-        element = etree.Element(self.ns + self.__name__)
-        if self.id:
-            element.set('id', self.id)
-        return element
-
-    def from_string(self, xml_string):
-        self.from_element(etree.XML(xml_string))
-
-    def from_element(self, element):
-        if self.ns + self.__name__ != element.tag:
-            raise TypeError
-        else:
-            if element.get('id'):
-                self.id = element.get('id')
 
 class Style(_StyleSelector):
     """
@@ -115,7 +93,7 @@ class Style(_StyleSelector):
 
     def styles(self):
         for style in self._styles:
-            if isinstance(style, _ColorStyle):
+            if isinstance(style, (_ColorStyle, BalloonStyle)):
                 yield style
             else:
                 raise TypeError
@@ -142,6 +120,7 @@ class Style(_StyleSelector):
             thestyle = LabelStyle(self.ns)
             thestyle.from_element(style)
             self.append_style(thestyle)
+        #XXX BalloonStyle
 
     def etree_element(self):
         element = super(Style, self).etree_element()
@@ -191,7 +170,6 @@ class StyleMap(_StyleSelector):
                     normal = StyleUrl(self.ns)
                     normal.from_element(style_url)
                 else:
-                    import ipdb; ipdb.set_trace()
                     raise ValueError
                 self.normal = normal
             else:
@@ -216,7 +194,7 @@ class StyleMap(_StyleSelector):
         return element
 
 
-class _ColorStyle(object):
+class _ColorStyle(_BaseObject):
     """
     abstract element; do not create.
     This is an abstract element and cannot be used directly in a KML file.
@@ -238,19 +216,13 @@ class _ColorStyle(object):
     # A value of random applies a random linear scale to the base <color>
 
     def __init__(self, ns=None, id=None, color=None, colorMode=None):
-        self.id = id
-        if ns == None:
-            self.ns = config.NS
-        else:
-            self.ns = ns
+        super(_ColorStyle, self).__init__(ns, id)
         self.color = color
         self.colorMode = colorMode
 
 
     def etree_element(self):
-        element = etree.Element(self.ns + self.__name__)
-        if self.id:
-            element.set('id', self.id)
+        element = super(_ColorStyle, self).etree_element()
         if self.color:
             color = etree.SubElement(element, "%scolor" %self.ns)
             color.text = self.color
@@ -259,8 +231,6 @@ class _ColorStyle(object):
             colorMode.text = self.colorMode
         return element
 
-    def from_string(self, xml_string):
-        self.from_element(etree.XML(xml_string))
 
     def from_element(self, element):
         if self.ns + self.__name__ != element.tag:
@@ -417,5 +387,8 @@ class LabelStyle(_ColorStyle):
         if scale is not None:
             self.scale = float(scale.text)
 
-def BalloonStyle(object):
+class BalloonStyle(_BaseObject):
+    """ Specifies how the description balloon for placemarks is drawn.
+        The <bgColor>, if specified, is used as the background color of
+        the balloon."""
     pass
